package com.tzld.piaoquan.recommend.model.produce.xgboost;

import com.aliyun.odps.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import ml.dmlc.xgboost4j.scala.spark.XGBoostClassificationModel;
import ml.dmlc.xgboost4j.scala.spark.XGBoostClassifier;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.ml.linalg.SparseVector;
import org.apache.spark.ml.linalg.VectorUDT;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.RowFactory;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.types.DataTypes;
import org.apache.spark.sql.types.StructField;
import org.apache.spark.sql.types.StructType;

import java.util.ArrayList;
import java.util.List;

/**
 * @author dyp
 */
@Slf4j
public class XGBoostTrain {
    public static void main(String[] args) {
        try {
            SparkSession spark = SparkSession.builder()
                    .appName("XGBoostTrain")
                    //.master("local")
                    .getOrCreate();

            JavaSparkContext jsc = new JavaSparkContext(spark.sparkContext());
            String file = "/dw/recommend/model/33_ad_train_data_v3_checkfeature/20240703/part-00099.gz";
            JavaRDD<String> rdd = jsc.textFile(file);

            // 将 RDD[LabeledPoint] 转换为 JavaRDD<Row>
            JavaRDD<Row> rowRDD = rdd.map(s -> {
                String[] line = StringUtils.split("\t");
                int label = NumberUtils.toInt(line[0]);
                int[] indices = new int[line.length - 1];
                double[] values = new double[line.length - 1];
                for (int i = 1; i < line.length; i++) {
                    String[] fv = StringUtils.split(":");
                    indices[i - 1] = i - 1;
                    values[i - 1] = NumberUtils.toDouble(fv[1], 0.0);
                }
                SparseVector vector = new SparseVector(indices.length, indices, values);
                return RowFactory.create(label, vector);
            });

            // log.info("rowRDD count {}", rowRDD.count());
            // 将 JavaRDD<Row> 转换为 Dataset<Row>
            List<StructField> fields = new ArrayList<>();
            fields.add(DataTypes.createStructField("label", DataTypes.IntegerType, true));
            fields.add(DataTypes.createStructField("features", new VectorUDT(), true));
            StructType schema = DataTypes.createStructType(fields);
            Dataset<Row> dataset = spark.createDataFrame(rowRDD, schema);


            // 划分训练集和测试集
            Dataset<Row>[] splits = dataset.randomSplit(new double[]{0.7, 0.3});
            Dataset<Row> trainData = splits[0];
            Dataset<Row> testData = splits[1];

            // 参数


            // 创建 XGBoostClassifier 对象
            XGBoostClassifier xgbClassifier = new XGBoostClassifier()
                    .setEta(0.1f)
                    .setFeaturesCol("features")
                    .setLabelCol("label")
                    .setMaxDepth(5)
                    .setObjective("binary:logistic")
                    .setNthread(1)
                    .setNumRound(10)
                    .setNumWorkers(2);


            // 训练模型
            XGBoostClassificationModel model = xgbClassifier.fit(trainData);

            // 显示预测结果
            Dataset<Row> predictions = model.transform(testData);
            predictions.select("label", "prediction").show();

        } catch (Exception e) {
            log.error("", e);
        }
    }
}
